﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Threading.Tasks;

namespace WpfApplication1
{
    public partial class MainWindow : Window
    {
        private TcpListener _TcpListener;
        private TcpClient _TcpClient;
        
        private bool _IsRun = false;

        protected NetworkStream _NetStream;

        public MainWindow()
        {
            InitializeComponent();
        }
        
        private void btnListener_Click(object sender, RoutedEventArgs e)
        {
            if (_IsRun)
                return;
            int port = Convert.ToInt32(txtPort.Text.Trim());
            _TcpListener = new TcpListener(new IPEndPoint(IPAddress.Any, port));
            _TcpListener.Start();
            _IsRun = true;

            Print("Listener (" + port + ") was started");

            new Task(() => {
                while (_IsRun)
                {
                    if (_TcpClient == null)
                    {
                        _TcpClient = _TcpListener.AcceptTcpClient();
                        Print("A new connected! remote:" + _TcpClient.Client.RemoteEndPoint.ToString());
                        _NetStream = _TcpClient.GetStream();
                    }
                    if (_TcpClient.Connected && _NetStream.DataAvailable)
                    {
                        byte[] buffer = null;
                        buffer = new byte[_TcpClient.Available];
                        _NetStream.Read(buffer, 0, buffer.Length);
                        string hexString = BitConverter.ToString(buffer, 0, buffer.Length).Replace("-", "");
                        string asciiString = ""; Encoding.ASCII.GetString(buffer, 0, buffer.Length).Trim('\0');
                        Print("HEX:" + hexString + " ASCII:" + asciiString);
                        
                        Analysis(buffer);
                    }
                }
            }).Start();
            
        }

        /// <summary>
        /// Log
        /// </summary>
        /// <param name="msg"></param>
        private void Print(string msg)
        {
            msg = System.DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss ffff") + "： " + msg + "\r";
            this.rtbLog.Dispatcher.Invoke(new Action(() =>
            {
                this.rtbLog.AppendText(msg);
                this.rtbLog.ScrollToEnd();
            }));
        }

        #region
        
        public void Analysis(byte[] data)
        {
            string log = "Analysis error";
            try
            {
                #region AVL11
                TZONE.AVL11.Protocols.Generic generic = new TZONE.AVL11.Protocols.Generic().Analysis(data);
                if (generic != null)
                {
                    log = "IMEI：" + generic.IMEI + " => Analysis Success";
                    if (!string.IsNullOrEmpty(generic.TagIDList)) 
                    {
                        List<TZONE.AVL11.Protocols.TAG> taglist = new TZONE.AVL11.Protocols.TAG().ConvertTag06(generic.TagIDList);
                        if (taglist != null && taglist.Count() > 0) 
                        {
                            foreach (var item in taglist)
                                log += "\r SN:" + item.SN + "Temperature:" + item.Temperature + " Humidity:" + item.Humidity;
                        }
                    }
                         
                }
                #endregion

            }
            catch (Exception) { }
            Print(log);
        }
        #endregion

    }
}
